package org.lossywav.algorithm;

// ==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//    Copyright (C) 2008, C# port by Greg Chudov.
//    Copyright (C) 2008, Java port by Josep Maria Antoln
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com

import org.lossywav.util.Constants;
/**
 * This class contains the precalculated analysis values for a given samplerate anb bitsize.
 * @author JosepMa
 */
public class FFTRecord {
	public double fft_underlap_length;
	public double bin_width;
	public int analysis_blocks;
	public int end_overlap_length;
	public int actual_analysis_blocks_start;
	public int max_bin;
	public double oneOverNumbins;
	public int lo_bin;
	public int hi_bin;

	public void generate(int currentBitLength, int sampleRate, double frequencyLimit, 
			int codecBlockSize, int underlap) {
		assert sampleRate>0 && currentBitLength > 0;
		final int numSamples = 1 << currentBitLength;
		final int halfSamples = 1 << (currentBitLength - 1);

		// Calculate actual analysis_time values for fft_lengths
		bin_width = sampleRate * 1.0 / numSamples;
		final double bin_time = 1.0 / bin_width;

		// Calculate which FFT bin corresponds to the low frequency limit
		lo_bin = (int) Math.max(1,	Math.round(Constants.LOW_FREQ_LIMIT * bin_time) - 1);
		hi_bin = (int) Math.max(1,	Math.round(frequencyLimit * bin_time) - 1);
		
		end_overlap_length = Math.min(halfSamples, codecBlockSize);
		actual_analysis_blocks_start = -end_overlap_length;
		final int total_overlap_length = codecBlockSize + end_overlap_length * 2 - numSamples;
		/// ???????
	    fft_underlap_length = Math.max(1, numSamples / (Math.pow(2.0,underlap)));
        analysis_blocks = Math.max(0, (int)Math.floor(total_overlap_length/fft_underlap_length));
	    fft_underlap_length=total_overlap_length/Math.max(1,analysis_blocks);
		
		max_bin = hi_bin - 1;
		oneOverNumbins = 1.0/ (max_bin - lo_bin + 1.0);
	}
}
