package org.lossywav.algorithm;

// ==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//    Copyright (C) 2008, C# port by Greg Chudov.
//    Copyright (C) 2008, Java port by Josep Maria Antoln
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com

import org.lossywav.util.Constants;

/**
 * This class contains the precalculated analysis values for a given samplerate
 * anb bitsize.
 * 
 * @author JosepMa
 */
public class SpreadingRecord {

	public static class SpreadingNew {
		public float width;
		public float divisor;
	}

	public class SpreadingOld {
		public int width;
		public int fraction;
	}

	public SpreadingOld[] spreadingAverage;
	public byte[] thresholdIndex;
	//TODO: This variable doesn't really belong here. It maps a precaculated analysis to a specific bit length for the fftRecord
	public int fft_bit_length;

	public SpreadingRecord() {
		thresholdIndex = new byte[Constants.THRESHOLD_INDEX_SPREAD_MAX];
	}

	public void generate(int index, int currentBitLength,
			double frequencyLimit, FFTRecord fftRecord) {
		assert (currentBitLength > 0);
	
		final int halfSamples = 1 << (currentBitLength - 1);
		this.fft_bit_length = currentBitLength;

		spreadingAverage = new SpreadingOld[1 << ( currentBitLength - 1 ) + 2];
		for (int i = 0; i < spreadingAverage.length ; i++) {
			spreadingAverage[i] = new SpreadingOld();
		}
		final int vl_bin = (int) Math.max(1,
				Math.round(Constants.VAR_LO_FREQUENCY[index] / fftRecord.bin_width)	- 1);
		int last_filled = 0;
		for (int bit = 0; bit < Constants.ANALYSIS_BITS_TO_CALCULATE; bit++) {
			// TODO: Could use the precalculated table again?
			double this_reference_threshold = (
					(currentBitLength * 0.5 + (bit + 1) - Constants.NOISE_CALC_OFFSET)
					* Constants.LOG2X20
					+ 10 * Math.log10(1 + (1 / Math.pow(2.0, 2 * bit + 1)))
					) * Constants.THRESHOLD_INDEX_SPREAD;
			while (last_filled < this_reference_threshold) {
				thresholdIndex[last_filled++] = (byte) bit;
			}
		}

		while (last_filled < Constants.THRESHOLD_INDEX_SPREAD_MAX) {
			thresholdIndex[last_filled++] = Constants.ANALYSIS_BITS_TO_CALCULATE - 1;
		}

		// Calculate which FFT bin corresponds to the low frequency limit
		int[] frequency_bins = new int[Constants.SPREAD_ZONES + 1];
		for (int i = 0; i < Constants.SPREAD_ZONES; i++) {
			frequency_bins[i] = Math.min(halfSamples, Math
					.max(1, (int) Math.round(Constants.FREQUENCY_LIMITS[i]
							/ fftRecord.bin_width))) - 1;
			if (frequency_bins[i] > halfSamples - 1) {
				// LossyWav_Error('frequency too high : '+floattostrf(FREQUENCY_LIMITS[sa_i]/1000,ffFixed,99,3)+'kHz',$21);
			}
		}
		frequency_bins[Constants.SPREAD_ZONES] = Math
				.min(halfSamples, Math.max(1, (int) Math.round(frequencyLimit
						/ fftRecord.bin_width))) - 1;

		for (int j = Constants.SPREAD_ZONES - 1; j >= 0; j--) {
			for (int i = frequency_bins[j]; i <= frequency_bins[j + 1]; i++) {
				spreadingAverage[i].width = Math.min(
						Math.max(2,	fftRecord.hi_bin - i + 1),
						Constants.SPREADING_FUNCTION_ARRAY[index][j]);
			}
		}
		for (int i = 0; i < vl_bin; i++) {
			spreadingAverage[i].fraction = 0;
		}
		for (int i = vl_bin; i <= fftRecord.hi_bin; i++) {
			spreadingAverage[i].fraction = (int) Math.floor(Constants.SPREADING_STEPS
							* Math.max(0,
										Math.min((frequencyLimit - Constants.VAR_LO_FREQUENCY[index]),
												i* fftRecord.bin_width	- Constants.VAR_LO_FREQUENCY[index]))
							/ (frequencyLimit - Constants.VAR_LO_FREQUENCY[index])
							* Constants.SPREADING_WIDTH[index]);

		}
	}
}
