package org.lossywav.audiocodecs;
//==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//    Copyright (C) 2008, C# port by Greg Chudov.
//    Copyright (C) 2008, Java port by Josep Maria Antoln
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com
import java.io.IOException;
import java.util.Map;
/**
 * Interface for an Audio output element.
 * @author JosepMa
 *
 */
public interface AudioDest {
	/**
	 * Get the path or description of this output element.
	 * @return String
	 */
	String getPath();
	/**
	 * Write metadata (tags) to the output.
	 * @param tags Map of tag name and tag content.
	 */
	void setTags(Map<String, String> tags);
	//void setFinalSampleCount(long finalSampleCount);
	/**
	 * Close the output, writing any data pending, and free any resources used.
	 * @throws IOException
	 */
	void close() throws IOException;
	/**
	 * Close and delete the file from the disk, if applicable.
	 * @throws IOException
	 */
	void delete() throws IOException;
	/**
	 * Write audio data to the output.
	 * @param buffer array with [channel][sample] of type int with the sample values
	 * @param sampleCount int amount of samples to read from the buffer
	 * @throws IOException any problem related to writing to the file
	 */
	void write(int[][] buffer, int sampleCount) throws IOException;
}
