package org.lossywav.audiocodecs;
//==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//    Copyright (C) 2008, C# port by Greg Chudov.
//    Copyright (C) 2008, Java port by Josep Maria Antoln
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com
import java.io.IOException;
import java.util.Map;
/**
 * Interface for an Audio input element.
 * @author JosepMa
 *
 */
public interface AudioSource {
	/**
	 * Get the bitwidth of each sample of this audio source.
	 * @return int
	 */
	int getBitsPerSample();
	/**
	 * Get the number of audio channels of this audio source.
	 * @return int
	 */
	int getChannelCount();
	/**
	 * Get the sampling rate of the samples of this audio source.
	 * @return int
	 */
	int getSampleRate();
	/**
	 * Get the length in samples of this audio source, if applicable.
	 * @return int
	 */
	long getLength();
	/**
	 * Get the path or description of this input element.
	 * @return String
	 */
	String getPath();
	/**
	 * Get the position in samples of the cursor.
	 * @return long
	 */
	long getPosition();
	/**
	 * Set the position in samples of the cursor.
	 * @param position long
	 * @throws IOException
	 */
	void setPosition(long position) throws IOException;
	/**
	 * Get the amount of samples remaining to be read ( length - position ).
	 * @return long
	 */
	long getRemaining();
	/**
	 * Obtain the metadata tags of this audio source.
	 * @return tags Map of tag name and tag content.
	 */
	Map<String, String> getTags();
	/**
	 * Write metadata (tags) to the output.
	 * @param tags Map of tag name and tag content.
	 */
	void setTags(Map<String, String> tags);
	/**
	 * Read samples from the audio source.
	 * @param buffer array of [channel][samples] with the samples
	 * @param sampleCount amount of samples to get.
	 * @return the real amount of samples read. Should be the same than sampleCount if everything is ok
	 * @throws IOException
	 */
	int read(int[][] buffer, int sampleCount) throws IOException;
	/**
	 * Close the output, writing any data pending, and free any resources used.
	 * @throws IOException
	 */
	void close() throws IOException;
}