package org.lossywav.audiocodecs;
//==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//    Copyright (C) 2008, C# port by Greg Chudov.
//    Copyright (C) 2008, Java port by Josep Maria Antoln
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com
import java.io.IOException;
import java.util.Map;
/**
 * Dummy example of an audio source (generates silence).
 * @author JosepMa
 *
 */
public class SilenceGenerator implements AudioSource {
	/**
	 * bitswidth of a sample of a single channel.
	 */
	private static final int bitsPerSample = 16;
	/**
	 * number of channels.
	 */
	private static final int channelCount = 2;
	/**
	 * sampling rate of the samples.
	 */
	private static final int sampleRate = 44100;
	/**
	 * length of this source.
	 */
	private final long length;
	/**
	 * Position in the stream.
	 */
	private long position;
	/**
	 * SilenceGenerator constructor.
	 * @param length int with the amount of samples to report as length.
	 */
	public SilenceGenerator(int length) {
		position = 0;
		this.length = length;
	}
	/**
	 * Get the bitwidth of each sample of this audio source.
	 * @return int
	 */
	public int getBitsPerSample() {
		return bitsPerSample;
	}
	/**
	 * Get the number of audio channels of this audio source.
	 * @return int
	 */
	public int getChannelCount() {
		return channelCount;
	}
	/**
	 * Get the sampling rate of the samples of this audio source.
	 * @return int
	 */
	public int getSampleRate() {
		return sampleRate;
	}
	/**
	 * Get the length in samples of this audio source, if applicable.
	 * @return int
	 */
	public long getLength() {
		return length;
	}
	/**
	 * Get the path or description of this input element.
	 * @return String
	 */
	public String getPath() {
		return null;
	}
	/**
	 * Get the position in samples of the cursor.
	 * @return long
	 */
	public long getPosition() {
		return position;
	}
	/**
	 * Set the position in samples of the cursor.
	 * @param position long
	 * @throws IOException
	 */
	public void setPosition(long position) {
		this.position = position;
	}
	/**
	 * Get the amount of samples remaining to be read ( length - position ).
	 * @return long
	 */
	public long getRemaining() {
		return length - position;
	}
	/**
	 * Obtain the metadata tags of this audio source.
	 * @return tags Map of tag name and tag content.
	 */
	public Map<String, String> getTags() {
		// TODO Auto-generated method stub
		return null;
	}
	/**
	 * Write metadata (tags) to the output.
	 * @param tags Map of tag name and tag content.
	 */
	public void setTags(Map<String, String> tags) {
		// TODO Auto-generated method stub

	}
	/**
	 * Read samples from the audio source.
	 * @param buffer array of [channel][samples] with the samples
	 * @param sampleCount amount of samples to get.
	 * @return the real amount of samples read. Should be the same than sampleCount if everything is ok
	 * @throws IOException
	 */
	public int read(int[][] buffer, int sampleCount) {
		final long samplesRemaining = getRemaining();
		if ( sampleCount > samplesRemaining ) {
			sampleCount = (int) samplesRemaining;
		}
		for(int i = 0 ; i < sampleCount; i++) {
			for(int j = 0; j < buffer[0].length; j++ ) {
				buffer[i][j] = 0;
			}
		}
		setPosition(getPosition() + sampleCount);
		return sampleCount;
	}
	public void close() {
	}

}
