package org.lossywav.dsp;
//==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//    Copyright (C) 2008, C# port by Greg Chudov.
//    Copyright (C) 2008, Java port by Josep Maria Antoln
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com
/**
 * ComplexBuffer implemented as an Array of type Complex[]
 * @author JosepMa
 *
 */
public class ComplexArrayBuffer implements ComplexBuffer {
	Complex[] buffer;

	public ComplexArrayBuffer(int bufferSize) {
		buffer =  new Complex[bufferSize];
	}
	
	public Complex getComplex(int position) {
		return buffer[position];
	}

	
	public void setComplex(int position, Complex complex) {
		buffer[position] = complex;
	}

	
	public void setComplex(int position, double real, double img) {
		buffer[position] = new Complex(real, img);
	}
	
	public double getImag(int position) {
		return buffer[position].im();
	}

	
	public void setImag(int position, double imag) {
		buffer[position] =  new Complex(buffer[position].re(), imag);
	}

	
	public double getReal(int position) {
		return buffer[position].re();
	}

	
	public void setReal(int position, double real) {
		buffer[position] = new Complex(real, buffer[position].im());
	}

	
	public void add(int positionA, Complex complexB) {
		buffer[positionA] = buffer[positionA].plus(complexB);
	}
	
	public void add(int positionA, double real, double imaginary) {
		buffer[positionA] = buffer[positionA].plus(new Complex(real, imaginary));
	}
	
	public void substract(int positionA, Complex complexB) {
		buffer[positionA] = buffer[positionA].minus(complexB);
	}
	
	public void substract(int positionA, double real, double imaginary) {
		buffer[positionA] = buffer[positionA].minus(new Complex(real, imaginary));

	}
	
	public void multiply(int positionA, Complex complexB) {
		buffer[positionA] = buffer[positionA].times(complexB);
	}

	
	public void multiply(int positionA, double realB) {
		buffer[positionA] = buffer[positionA].times(realB);
	}

	
	public Complex sumPositions(int positionA, int positionB) {
		return buffer[positionA].plus(buffer[positionB]);
	}

	
	public void sumPositionsInA(int positionA, int positionB) {
		buffer[positionA] = buffer[positionA].plus(buffer[positionB]);
	}
	
	public Complex substractPositions(int positionA, int positionB) {
		return buffer[positionA].minus(buffer[positionB]);
	}
	
	public void substractPositionsInA(int positionA, int positionB) {
		buffer[positionA] = buffer[positionA].minus(buffer[positionB]);
	}
	
	public Complex multiplyPositions(int positionA, int positionB) {
		return buffer[positionA].times(buffer[positionB]);
	}
	
	public void multiplyPositionsInA(int positionA, int positionB) {
		buffer[positionA] = buffer[positionA].times(buffer[positionB]);
	}

	
	public void swapPositions(int positionA, int positionB) {
		final Complex temp = buffer[positionA];
		buffer[positionA] = buffer[positionB];
		buffer[positionB] = temp;
	}

	
	public double multiplyComplexWithPositionAndReturnImag(Complex complexA,
			int positionB) {
		return complexA.times(buffer[positionB]).im();
	}
	
	public double multiplyComplexWithPositionAndReturnReal(Complex complexA,
			int positionB) {
		return complexA.times(buffer[positionB]).re();
	}
	
	public double absolute(int positionA) {
		return buffer[positionA].abs();
	}



}
