package org.lossywav.dsp;
//==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//    Copyright (C) 2008, C# port by Greg Chudov.
//    Copyright (C) 2008, Java port by Josep Maria Antoln
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com
/**
 * This source is taken from http://www.dsptutor.freeuk.com/
 * and slightly modified.
 */
public class WindowFunction {
	/**
	 * Indicator of window type to generate.
	 */
	private WindowingType windowType = WindowingType.RECTANGULAR;
	/**
	 * setter of windowType
	 * @param wt
	 */
	public void setWindowType(WindowingType wt) {
		windowType = wt;
	}
	/**
	 * getter of windowType
	 * @return
	 */
	public WindowingType getWindowType() {
		return windowType;
	}
	/**
	 * Generate nSamples window function values for index values 0 .. nSamples - 1.
	 * @param nSamples Amount of samples.
	 * @return array of floats normalized to 1.0f
	 */
	//TODO: Generate correctly for odd and even nSamples, i.e. to be two equal halves.
	public float[] generate(int nSamples, float scale) {
		final int nHalf = nSamples / 2;
		float r;
		final float pi = (float) Math.PI;
		final float[] w = new float[nSamples];
		switch (windowType) {
		case BARTLETT:
			for (int n = 0; n < nSamples; n++) {
				w[n] =( 1.0f - (float) Math.abs(n - nHalf) / (float) nHalf) * scale;
			}
			break;
		case HANN:
			r = pi / nHalf;
			for (int n = -nHalf; n < nHalf; n++) {
				// n + 0.5 added to be a mirrored function with an even amount of samples.
				//w[nHalf + n] = (0.5f + 0.5f * (float) Math.cos((n + 0.5) * r)) * scale;
				w[nHalf + n] = (0.5f + 0.5f * (float) Math.cos((n) * r)) * scale;
			}
			break;
		case HAMMING:
			r = pi / nHalf;
			for (int n = -nHalf; n < nHalf; n++) {
				// n + 0.5 added to be a mirrored function with an even amount of samples.
				w[nHalf + n] = (0.54f + 0.46f * (float) Math.cos((n + 0.5) * r)) * scale;
			}
			break;
		case BLACKMAN:
			r = pi / nHalf;
			for (int n = -nHalf; n < nHalf; n++) {
				// n + 0.5 added to be a mirrored function with an even amount of samples.
				w[nHalf + n] = (0.42f + 0.5f * (float) Math.cos((n + 0.5) * r) +
						0.08f * (float) Math.cos(2 * (n + 0.5) * r)) * scale;
			}
			break;
		default:
			for (int n = 0; n < nSamples; n++) {
				w[n] = scale;
			}
		}
		return w;
	}
}