package org.lossywav.util;

public class CircularBuffer4 {
	private final int[] blockPositionLookupBlock;
	private final int[] blockPositionLookupOffset;

	private int[][][] buffer;
	private int[] length;
	public CircularBuffer4(int bufferSize, int numChannels) {
		blockPositionLookupBlock = new int[bufferSize * ReadBlockPosition.values().length];
		blockPositionLookupOffset = new int[bufferSize * ReadBlockPosition.values().length];
		for (int i = 0; i < bufferSize * ReadBlockPosition.values().length; i++) {
			blockPositionLookupBlock[i] = i / bufferSize;
			blockPositionLookupOffset[i] = i % bufferSize;
		}
		buffer = new int[ReadBlockPosition.values().length][numChannels][bufferSize];
		length = new int[ReadBlockPosition.values().length];
	}
	public void rotate() {
		final int[][] block = buffer[ReadBlockPosition.TWOPREVIOUS.index];

		buffer[ReadBlockPosition.TWOPREVIOUS.index] = buffer[ReadBlockPosition.PREVIOUS.index];
		buffer[ReadBlockPosition.PREVIOUS.index] = buffer[ReadBlockPosition.CURRENT.index];
		buffer[ReadBlockPosition.CURRENT.index] = buffer[ReadBlockPosition.NEXT.index];
		buffer[ReadBlockPosition.NEXT.index] = block;

		length[ReadBlockPosition.TWOPREVIOUS.index] = length[ReadBlockPosition.PREVIOUS.index];
		length[ReadBlockPosition.PREVIOUS.index] = length[ReadBlockPosition.CURRENT.index];
		length[ReadBlockPosition.CURRENT.index] = length[ReadBlockPosition.NEXT.index];
		length[ReadBlockPosition.NEXT.index] = 0;

	}
	public int[][] getCurrentBuffer() {
		return buffer[ReadBlockPosition.CURRENT.index];
	}
	public int[][] getNextBuffer() {
		return buffer[ReadBlockPosition.NEXT.index];
	}
	public int getCircularSample(int channel, int sampleIdx) {
		return buffer[blockPositionLookupBlock[sampleIdx]][channel][blockPositionLookupOffset[sampleIdx]];
	}
	public int getChannelSample(int channel, int sampleIdx) {
		assert (channel < buffer[ReadBlockPosition.CURRENT.index].length);
		assert (sampleIdx < length[ReadBlockPosition.CURRENT.index]);
		return buffer[ReadBlockPosition.CURRENT.index][channel][sampleIdx];
	}
	public int getLength(int which) { 
		assert (which < ReadBlockPosition.values().length);
		return length[which];
	}
	/**
	 * Get the length of actual audio data.
	 * @return
	 */
	public int getCurrentLength() {
		return length[ReadBlockPosition.CURRENT.index];
	}
	/**
	 * set the length of actual audio data.
	 * @param length
	 */
	public void setNextLength(int length) {
		assert (length > buffer[ReadBlockPosition.NEXT.index][0].length);
		this.length[ReadBlockPosition.NEXT.index] = length;
	}
}
