package org.lossywav.util;
//==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//    Copyright (C) 2008, C# port by Greg Chudov.
//    Copyright (C) 2008, Java port by Josep Maria Antoln
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com
/**
 * Class of constants.
 * @author JosepMa
 */
public class Constants {
	// Tags
	public static final String LOSSYWAV_TAG = "lossyWAV 1.1.4";
	public static final String JLOSSYWAV_TAG = "jLossyWav 0.5.0";

	// Ranges and default Parameters.
	public static final double MIN_QUALITY = -4.0;
	public static final double DEFAULT_QUALITY = 5.0;
	public static final double MAX_QUALITY = 10.0;
	
	public static final double MIN_SCALING = 0.00001;
	public static final double DEFAULT_SCALING = 1.0;
	public static final double MAX_SCALING = 8.0;

	public static final int MIN_UNDERLAP = 1;
	public static final int DEFAULT_UNDERLAP = 1;
	public static final int MAX_UNDERLAP = 8;	

	public static final int MIN_PARAM_ANALYSIS = 2;
	public static final int DEFAULT_PARAM_ANALYSIS = 2;
	public static final int MAX_PARAM_ANALYSIS = 5;
	
	public static final int MAX_CLIPS = 16;
	public static final int MIN_CLIPS = 0;

	// Default is quality/10;
	public static final double MIN_SHAPING = 0.0;
	public static final double MAX_SHAPING = 1.0;
	
	// Default is 0, i.e. no delta
	public static final double MIN_DELTANTS = -50.0;
	public static final double MAX_DELTANTS = 50.0;

	// Default is 0, i.e. no delta
	public static final double MIN_DELTASNR = -50.0;
	public static final double MAX_DELTASNR = 50.0;
	
	public static final double LOW_FREQ_LIMIT = 20.0;
	public static final double MID_FREQ_BAND_SKEWING = 3445.3125;

	public static final float MIN_UPPER_FREQUENCY_LIMIT = 16000F;
	public static final float DEFAULT_UPPER_FREQUENCY_LIMIT = 16000F;
	public static final float MAX_UPPER_FREQUENCY_LIMIT = 20000F;

	public static final boolean DEFAULT_FORCE_FFT32 = false; // --impulse
	public static final boolean DEFAULT_LINK_CHANNELS = false;
	public static final boolean DEFAULT_MID_SIDE = false;
	public static final boolean DEFAULT_SORTSPREAD = false;

	// Quality Related arrays
	public static final double[] QUALITY_NOISE_THRESHOLDS_SHIFT = {36, 32, 28, 24, 20, 16, 9, 6, 3, 0, -2.4, -4.8, -7.2, -9.6, -12};
	public static final double[] QUALITY_NOISE_THRESHOLDS_SHIFT_DELTA = {2.874, 2.633, 2.402, 2.197, 1.961, 1.728, 2.674, 2.384, 2.716, 2.750, 2.639, 2.525, 2.366, 2.227, 2.012};
	public static final double[] QUALITY_SIGNAL_TO_NOISE_RATIO = {-2, -6, -10, -14, -18, -22, -23.5, -23.5, -23.5, -25, -28, -31, -34, -37, -40};
	public static final double[] QUALITY_DYNAMIC_MINIMUM_BITS_TO_KEEP = {1.50, 1.75, 2.00, 2.25, 2.50, 2.75, 3.00, 3.25, 3.50, 3.75, 4.00, 4.25, 4.50, 4.75, 5.00};
	public static final double[] QUALITY_MAXIMUM_CLIPS_PER_CHANNEL = { 3, 3, 3, 3, 3, 3, 3, 3, 2, 1, 0, 0, 0, 0, 0 };
	public static final boolean[] QUALITY_AUTO_FFT32_ON = { false,  false, false, false, false, false, false, true, true, true, true, true, true, true, true };

	public static final int[] PRECALC_ANALYSIS_BITS_DIFF = { -4 , -3 , -2, -1, 0, 1 };
	
	public static final double[] VAR_LO_FREQUENCY = { 20, 20, 20, 1378.125, 1378.125, 3445.3125 };
	public static final float[] FREQUENCY_LIMITS = { 20F, 1378.125F, 3445.3125F, 5512.5F, 8268.75F, 10335.9375F, 12403.125F, 14470.3125F}; //highest one is the lowpass
	public static final float[] SORTSPREAD_MULTIPLIERS = { 0.421350947020f, 0.276953179437f, 0.182040800295f, 0.119655073248f };
	public static final double[] SPREADING_WIDTH = {0.250, 0.176777, 0.250, 0.353554, 0.500, 0.707107};
	public static final int[][] SPREADING_FUNCTION_ARRAY = {{2,2,2,2,2,2,2,2},{2,2,2,2,2,2,2,2},{2,2,2,2,2,2,2,3},
		{1,2,2,2,2,2,2,3},{1,2,2,2,2,2,2,4},{1,1,2,2,2,2,3,4}};

	public static final double[] order_4_48000_a = { +0.90300, +0.01160, -0.58530, -0.25710, 0, 0 };
	public static final double[] order_4_48000_b = { -2.23740, +0.73390, +0.12510, +0.60330, 0, 0 };
	public static final double[] order_4_44100_a = { +1.05870, +0.06760, -0.60540, -0.27380, 0, 0 };
	public static final double[] order_4_44100_b = { -2.20610, +0.47070, +0.25340, +0.62130, 0, 0 };

	
	// Analysis Constants
	public static final int MAX_FFT_BITWIDTH = 13;
	public static final int MAX_FFT_LENGTH = 1 << MAX_FFT_BITWIDTH; // 8192
	public static final int PRECALC_ANALYSES = PRECALC_ANALYSIS_BITS_DIFF.length;
	public static final int ANALYSIS_BITS_TO_CALCULATE  = 32;
	
	public static final short MINIMUM_BITS_TO_KEEP = 6;
	//TODO: REALLY 7?
	public static final int SPREAD_ZONES = 8;
	public static final int SPREADING_STEPS  = 16384;
	public static final int THRESHOLD_INDEX_SPREAD = 64;
	public static final int THRESHOLD_INDEX_SPREAD_MUL = 256;
	public static final int THRESHOLD_INDEX_SPREAD_MAX = THRESHOLD_INDEX_SPREAD * THRESHOLD_INDEX_SPREAD_MUL;
	public static final double NOISE_CALC_OFFSET = 2.91639752890751;
	
	public static final int SKEWING_AMPLITUDE = 36;
	public static final int SHAPING_LENGTH = order_4_48000_a.length;
	
	public static final double MAX_DB = 5623413251903.490803949510; // { 10^(255/20) }

	//Input/Output
	//TODO: Verify the correct use of this constant.
	public static final int MAX_BLOCK_SAMPLES = 4096;
	public static final int MAX_CHANNELS = 8;
	public static final int BUFFER_SIZE = MAX_CHANNELS * MAX_BLOCK_SAMPLES * 4; // 4 is maxbytewidth

/*	public static final float[][] REFERENCE_THRESHOLD = { // From bit 5, to bit 13.
		{ 0.855049F,1.647705F,2.586367F,3.570348F,4.566311F,5.565302F, 6.565059F, 7.565002F, 8.564982F, 9.564947F,10.564983F,11.564966F,12.564990F,13.564963F,14.564963F,15.564953F,16.564938F,17.564932F,18.564946F,19.564977F,20.564993F,21.564943F,22.564971F,23.564972F,24.564971F,25.564942F,26.564973F,27.564952F,28.564934F,29.564964F,30.565000F,31.564957F},
		{ 1.365595F,2.158121F,3.096140F,4.079761F,5.075650F,6.074656F, 7.074348F, 8.074301F, 9.074286F,10.074265F,11.074260F,12.074282F,13.074297F,14.074298F,15.074254F,16.074310F,17.074259F,18.074278F,19.074298F,20.074277F,21.074247F,22.074304F,23.074269F,24.074309F,25.074261F,26.074301F,27.074316F,28.074259F,29.074281F,30.074272F,31.074257F,32.074265F},
		{ 1.870837F,2.663372F,3.600974F,4.584524F,5.580366F,6.579303F, 7.579018F, 8.578990F, 9.578999F,10.578921F,11.578967F,12.578961F,13.578965F,14.578950F,15.578952F,16.578964F,17.578957F,18.578957F,19.578943F,20.578949F,21.578953F,22.578919F,23.578966F,24.578940F,25.578934F,26.578952F,27.578921F,28.578943F,29.578934F,30.578939F,31.578957F,32.578939F},
		{ 2.373478F,3.165937F,4.103403F,5.086839F,6.082678F,7.081625F, 8.081372F, 9.081305F,10.081304F,11.081288F,12.081279F,13.081282F,14.081269F,15.081277F,16.081299F,17.081315F,18.081269F,19.081260F,20.081285F,21.081299F,22.081287F,23.081292F,24.081301F,25.081300F,26.081281F,27.081274F,28.081268F,29.081285F,30.081311F,31.081289F,32.081297F,33.081284F},
		{ 2.874786F,3.667300F,4.604600F,5.588036F,6.583839F,7.582784F, 8.582545F, 9.582496F,10.582470F,11.582446F,12.582467F,13.582453F,14.582474F,15.582478F,16.582438F,17.582437F,18.582458F,19.582484F,20.582439F,21.582456F,22.582444F,23.582433F,24.582435F,25.582454F,26.582425F,27.582443F,28.582443F,29.582451F,30.582450F,31.582509F,32.582402F,33.582447F},
		{ 3.375442F,4.167948F,5.105193F,6.088653F,7.084443F,8.083383F, 9.083136F,10.083083F,11.083088F,12.083042F,13.083045F,14.083021F,15.083045F,16.083047F,17.083032F,18.083045F,19.083051F,20.083031F,21.083038F,22.083041F,23.083036F,24.083033F,25.083059F,26.083044F,27.083043F,28.083031F,29.083039F,30.083038F,31.083027F,32.083062F,33.083057F,34.083017F},
		{ 3.875760F,4.668262F,5.605555F,6.588945F,7.584770F,8.583679F, 9.583467F,10.583333F,11.583352F,12.583357F,13.583353F,14.583342F,15.583322F,16.583328F,17.583336F,18.583304F,19.583327F,20.583344F,21.583317F,22.583326F,23.583307F,24.583321F,25.583300F,26.583365F,27.583308F,28.583330F,29.583333F,30.583336F,31.583339F,32.583320F,33.583304F,34.583342F},
		{ 4.375942F,5.168419F,6.105638F,7.089092F,8.084885F,9.083830F,10.083552F,11.083465F,12.083491F,13.083485F,14.083446F,15.083493F,16.083486F,17.083496F,18.083485F,19.083490F,20.083477F,21.083474F,22.083482F,23.083483F,24.083463F,25.083500F,26.083483F,27.083467F,28.083480F,29.083465F,30.083478F,31.083495F,32.083484F,33.083481F,34.083507F,35.083478F},
		{ 4.876022F,5.668487F,6.605748F,7.589148F,8.584943F,9.583902F,10.583678F,11.583589F,12.583571F,13.583568F,14.583551F,15.583574F,16.583541F,17.583544F,18.583555F,19.583554F,20.583574F,21.583577F,22.583531F,23.583560F,24.583548F,25.583570F,26.583561F,27.583581F,28.583552F,29.583540F,30.583564F,31.583530F,32.583546F,33.583585F,34.583557F,35.583567F}
	};
*/	


	
	//Math
	public static final double LOG2X20 = 6.0205999132796239042747778944899; // 20.0 * Math.log10(2.0)
	public static final double INVLOG2X2 = 1.6609640474436811739351597147447; // 1.0 / ( 2.0 * Math.log10(2.0))
	public static final double HALF_PI   = 1.570796326794896619231322;  // Pi/2
/*	  E             = 2.718281828459045235360287;  { e             }
	  TWO_PI         = 6.283185307179586476925287;  { 2*Pi          }
	  ONE_OVER_ROOT2  = 0.707106781186547524400844;  { 2^-0.5        }
	  PI            = 3.141592653589793238462643;  { Pi            }
	  LG_2           = 0.301029995663981195213739;  { lg10(2)       }
	  ONE_OVER_LG_2    = 3.321928094887362347870319;  { 1/lg10(2)     }
	  LN2           = 0.693147180559945309417232;  { ln(2)         }
	  LB_E           = 1.442695048889634073599247;  { lb(e)         }
	  MAX_DB        : Double = 5623413251903.490803949510;  { 10^(255/20)   }
	  GOLDEN_RATIO   = 1.618033988749894848204587;  { Golden Ratio  }
*/	
	
}
